<?php

namespace TrainingWheels\Environment;
use \TrainingWheels\Conn\ServerConn;

class LinuxEnv extends TrainingEnv {
  protected $conn;

  public function __construct(ServerConn $conn) {
    $this->conn = $conn;
    if (!$this->hasSudo()) {
      throw new \Exception('The connection needs to have root or sudo access to the server.');
    }
  }

  /**
   * Check if the currect connection has sudo access.
   */
  public function hasSudo() {
    $output = $this->conn->exec('sudo whoami');
    return $output === 'root';
  }

  /**
   * Append a text string to the end of a file.
   */
  public function fileAppendText($file_path, $text) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    if (!$this->fileExists($file_path)) {
      throw new \Exception("File $file_path not found in function " . __FUNCTION__);
    }
    $this->conn->exec("echo \"$text\" >> $file_path");
  }

  /**
   * Get the contents of a text file.
   */
  public function fileGetContents($file_path) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $out = $this->conn->exec("cat $file_path");
    if ($out == "cat: $file_path: No such file or directory") {
      throw new \Exception("Trying to get contents of file $file_path that does not exist.");
    }
    return $out;
  }

  /**
   * Put the contents of a text file, overwriting if one exists.
   */
  public function filePutContents($file_path, $contents) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $out = $this->conn->exec("echo \"$contents\" > $file_path");
    return $out;
  }

  /**
   * Use rsync to sync two folders. Purposefully abstracts a lot from rsync,
   * as otherwise this could do a lot of damage to the server.
   */
  public function fileSyncUserFolder($source_user, $target_user, $folder) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    if (!$this->userExists($source_user) || !$this->userExists($target_user)) {
      throw new \Exception("The user does not exist in function fileSyncUserFolder");
    }

    $source_path = "/home/$source_user/$folder";
    $target_path = "/home/$target_user/$folder";

    if ($source_path == $target_path) {
      throw new \Exception("Source and target cannot be equal: $source_path");
    }
    if (!$this->dirExists($source_path)) {
      throw new \Exception("Source directory does not exist $source_path");
    }

    $this->conn->exec("rsync -a --delete $source_path $target_path");
    $this->conn->exec("chown -R $target_user: $target_path");
  }

  /**
   * Recursively chmod a directory
   */
  public function dirChmod($options, $dir_path, $recurse = TRUE) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    if (!$this->dirExists($dir_path)) {
      throw new \Exception("Source directory does not exist $dir_path");
    }
    if ($recurse) {
      $options = '-R ' . $options;
    }
    $this->conn->exec("chmod $options $dir_path");
  }

  /**
   * Run sed on a file.
   */
  public function fileStrReplace($search, $replace, $file_path) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    if (!$this->fileExists($file_path)) {
      throw new \Exception("File does not exist: $file_path");
    }
    $this->conn->exec("sed -i'' -e's/$search/$replace/' $file_path");
  }

  /**
   * Create a text file.
   */
  public function fileCreate($text, $file_path, $user) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $this->conn->exec("echo $text > $file_path");
    $this->conn->exec("chown $user: $file_path");
  }

  /**
   * Delete a file.
   */
  public function fileDelete($file_path) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $this->conn->exec("rm $file_path");
  }

  /**
   * Copy a file.
   */
  public function fileCopy($source, $target) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $this->conn->exec("cp $source $target");
  }

  /**
   * Check if a Linux user exists in the system.
   */
  public function userExists($user) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $output = $this->conn->exec('grep "^' . $user . ':" /etc/passwd');
    return substr($output, 0, strlen($user) + 1) == $user . ':';
  }

  /**
   * Get all Linux users, just the ones with home directories.
   */
  public function usersGetAll() {
    $output = $this->conn->exec('ls /home');
    if (!empty($output)) {
      return explode("\n", $output);
    }
    else {
      return FALSE;
    }
  }

  /**
   * Check if a file exists in the system.
   */
  public function fileExists($file_path) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $output = $this->conn->exec("ls $file_path");
    return $output == $file_path;
  }

  /**
   * Check if a directory exists in the file system.
   */
  public function dirExists($dir_path) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $output = $this->conn->exec("test -d $dir_path && echo true");
    return $output == 'true';
  }

  /**
   * Create a user.
   */
  public function userCreate($user, $pass) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $conn = $this->conn;
    $conn->exec("groupadd $user");
    $conn->exec("rsync -avh --delete /opt/trainingwheels/skel/skel_user/ /tmp/skel_user/");
    $conn->exec("echo $pass > /tmp/skel_user/.password");
    $conn->exec("useradd -m -d/home/$user -k/tmp/skel_user -s/bin/bash -g$user $user");
    $conn->exec("echo $user:$pass | chpasswd");
    $conn->exec("chmod o-rwx /home/$user");
    $conn->exec("rm -rf /tmp/skel_user");
  }

  /**
   * Delete a user.
   */
  public function userDelete($user) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $this->conn->exec("userdel $user");
    $this->conn->exec("groupdel $user");
    $this->conn->exec("rm -rf /home/$user");
  }

  /**
   * Get a user id (Linux user id).
   */
  public function userGetId($user) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    if (!$this->userExists($user)) {
      throw new \Exception("The user '$user' does not exist, can't get id.");
    }
    $id = $this->conn->exec("id -u $user");
    return $id;
  }

  /**
   * Add a user to a group.
   */
  public function userAddToGroup($user, $group) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $this->conn->exec("gpasswd -a $user $group");
  }

  /**
   * Remove a user from a group.
   */
  public function userRemoveFromGroup($user, $group) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $this->conn->exec("gpasswd -d $user $group");
  }

  /**
   * Get a user's password.
   */
  public function userPasswdGet($user) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    if ($this->fileExists("/home/$user/.password")) {
      return $this->fileGetContents("/home/$user/.password");
    }
  }

  /**
   * Is the user logged in?
   */
  public function userIsLoggedIn($user) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $out = $this->conn->exec("users");
    $logged_in = explode(' ', $out);
    return in_array($user, $logged_in);
  }

  /**
   * Delete a MySQL database.
   */
  public function mySQLDBDelete($db) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $this->mySQLVerifyDBName($db);
    $this->conn->exec("echo \"DROP DATABASE $db;\" | mysql");
  }

  /**
   * Delete a MySQL database user.
   */
  public function mySQLUserDelete($user) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $this->mySQLVerifyUserName($user);
    $this->conn->exec("echo \"DROP USER '$user'@'localhost';\" | mysql");
  }

  /**
   * Create a MySQL database user.
   */
  public function mySQLUserCreate($user, $pass) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $this->mySQLVerifyUserName($user);
    $this->conn->exec("echo \"CREATE USER '$user'@'localhost' IDENTIFIED BY '$pass';\" | mysql");
  }

  /**
   * Create a MySQL database.
   */
  public function mySQLDBCreate($db) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $this->mySQLVerifyDBName($db);
    $this->conn->exec("echo \"CREATE DATABASE $db;\" | mysql");
  }

  /**
   * Grant privileges to a user for a database.
   */
  public function mySQLGrantAllToUser($db, $user) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $this->mySQLVerifyDBName($db);
    $this->mySQLVerifyUserName($user);
    $this->conn->exec("echo \"GRANT ALL PRIVILEGES on $db.* to '$user'@'localhost';\" | mysql");
  }

  /**
   * Dump a db to a file.
   */
  public function mySQLDumpToFile($db, $target_file, $gzip = TRUE) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $this->mySQLVerifyDBName($db);
    if (!$this->mySQLDBExists($db)) {
      throw new \Exception('Database does not exist on the server. Create one first.');
    }
    if ($gzip) {
      $db = $db . ' | gzip';
    }
    $this->conn->exec("mysqldump $db > $target_file");
  }

  /**
   * Import a database dump into a db.
   */
  public function mySQLImportFromDump($dump_path, $db, $gzip = TRUE) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $this->mySQLVerifyDBName($db);
    if (!$this->fileExists($dump_path)) {
      throw new \Exception('Database dump file does not exist on the server');
    }
    if (!$this->mySQLDBExists($db)) {
      throw new \Exception('Database does not exist on the server. Create one first.');
    }
    $cmd = $gzip ? 'zcat' : 'cat';
    $cmd = $cmd . " $dump_path | mysql $db";
    $this->conn->exec($cmd);
  }

  /**
   * Does a database exist?
   */
  public function mySQLDBExists($db) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $this->mySQLVerifyDBName($db);
    $cmd = "echo \"SELECT SCHEMA_NAME FROM INFORMATION_SCHEMA.SCHEMATA WHERE SCHEMA_NAME = '$db';\" | mysql -s";
    $output = $this->conn->exec($cmd);
    return $output === $db;
  }

  /**
   * Restart a service.
   */
  protected function serviceRestart($service) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $this->conn->exec("service $service restart");
  }

  /**
   * Clone the repo for a user.
   */
  public function gitRepoClone($user, $repo, $target) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    $this->conn->exec("git clone -q $repo $target");
    $this->conn->exec("chown -R $user: $target");
  }

  /**
   * Get the current branch of a git repo.
   */
  public function gitBranchGet($dir) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    return $this->conn->exec("cd $dir; git rev-parse --abbrev-ref HEAD");
  }

  /**
   * Check if there are local changes.
   */
  public function gitLocalChanges($dir) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    return $this->conn->exec("cd $dir; git st -s");
  }

  /**
   * Check what the remote is.
   */
  public function gitRemote($dir) {
    twcore_assert_valid_strings(__CLASS__ . '::' . __FUNCTION__, func_get_args());
    return $this->conn->exec("cd $dir; git remote -v");
  }
}