<?php

namespace TrainingWheels\Resource;

class MySQLDatabaseResource extends Resource {

  public $db_name;
  public $mysql_username;
  public $mysql_password;
  public $course;
  public $dump_path;

  /**
   * Constructor.
   */
  public function __construct(\TrainingWheels\Environment\TrainingEnv $env, $title, $user_name, $course, $dump_path) {
    parent::__construct($env, $title, $user_name);
    $this->course = $course;
    $this->dump_path = $dump_path;
  }

  /**
   * Get the info on this resource.
   */
  public function get() {
    $info = array(
      'type' => 'mysqldb',
      'exists' => $this->exists(),
      'title' => $this->title,
    );
    if ($info['exists']) {
      $info['attribs']['db_name'] = $this->dbNameGet();
      $info['attribs']['mysql_username'] = $this->userNameGet();
      $info['attribs']['mysql_password'] = $this->passwdGet();
    }
    return $info;
  }

  /**
   * Does the database exist in the environment?
   */
  public function exists() {
    return $this->env->mySQLDBExists($this->dbNameGet());
  }

  /**
   * Create the database.
   */
  public function create() {
    $this->env->mySQLUserCreate($this->userNameGet(), $this->passwdGet());
    $this->env->mySQLDBCreate($this->dbNameGet());
    $this->env->mySQLGrantAllToUser($this->dbNameGet(), $this->userNameGet());
    $this->env->mySQLImportFromDump($this->dump_path, $this->dbNameGet());
    $this->credentialsCreate();
  }

  /**
   * Delete the database.
   */
  public function delete() {
    $this->env->mySQLDBDelete($this->dbNameGet());
    $this->env->mySQLUserDelete($this->userNameGet());
  }

  /**
   * Make a .my.cnf file for easy access to DB.
   */
  protected function credentialsCreate() {
    $dbuser = $this->userNameGet();
    $pass = $this->passwdGet();
    $this->env->fileCreate("\"[client]\nuser=$dbuser\npass=$pass\n\"", "~$this->user_name/.my.cnf", $this->user_name);
  }

  /**
   * Create a name.
   */
  protected function genSafeDBName() {
    // Since MySQL has a limit of 16 chars for user names, we use the Linux
    // user id instead.
    $user_id = $this->env->userGetId($this->user_name);

    $name = $this->course . '_' . $user_id;

    // We turn all dashes into underscores, as dashes are illegal in DB names.
    $name = str_replace('-', '_', $name);

    return $name;
  }

  /**
   * Lazy generate the db name, which requires a linux user exist so can't
   * be done on construction.
   */
  public function dbNameGet() {
    if (!isset($this->db_name)) {
      $this->db_name = $this->genSafeDBName();
    }
    return $this->db_name;
  }

  /**
   * Lazy generate the username, which requires a linux user exist so can't
   * be done on construction.
   */
  public function userNameGet() {
    if (empty($this->mysql_username)) {
      $this->mysql_username = $this->genSafeDBName();
    }
    return $this->mysql_username;
  }

  /**
   * Lazy load the password from the credentials.
   */
  public function passwdGet() {
    if (empty($this->mysql_password)) {
      if ($this->env->fileExists("/home/$this->user_name/.my.cnf")) {
        $cnf = $this->env->fileGetContents("/home/$this->user_name/.my.cnf");

        if (!empty($cnf)) {
          $ini = parse_ini_string($cnf);
          $this->mysql_password = $ini['pass'];
        }
      }
    }
    return empty($this->mysql_password) ? twcore_passwd_gen() : $this->mysql_password;
  }

  /**
   * Dump contents to file.
   */
  public function dumpTo($file_folder) {
    $file_folder = trim($file_folder, '/');
    $db = $this->dbNameGet();
    $file = "/$file_folder/$db.sql.gz";
    $this->env->mySQLDumpToFile($db, $file);
    return $file;
  }

  /**
   * Sync to a target.
   */
  public function syncTo(MySQLDatabaseResource $target) {
    // Create a backup.
    if ($target->exists()) {
      $target->dumpTo('tmp');
    }

    static $source_dump = FALSE;
    if (!$source_dump) {
      $source_dump = $this->dumpTo('tmp');
    }

    $target->delete();
    $target->dump_path = $source_dump;
    $target->create();
  }
}
