<?php

namespace TrainingWheels\User;

abstract class User {

  // User name.
  protected $user_name;

  // Password.
  protected $password;

  // Reference to the training environment.
  protected $env;

  // Course name.
  protected $course;

  // Resources that this user needs.
  public $resources;

  // Does this user exist yet?
  protected $exists;

  /**
   * Constructor.
   */
  public function __construct(\TrainingWheels\Environment\TrainingEnv $env, $user_name, $course) {
    $this->env = $env;
    $this->user_name = $user_name;
    $this->course = $course;
  }

  /**
   * Create this user.
   */
  public function create() {
    $this->env->userCreate($this->user_name, $this->passwdGet());
  }

  /**
   * Delete this user.
   */
  public function delete() {
    // Remove the resources associated with this user first, as some may rely
    // on the user still existing in the system.
    foreach ($this->resources as $res) {
      $res->delete();
    }
    $this->env->userDelete($this->user_name);
  }

  /**
   * Return bool for whether the user exists in the environment.
   */
  public function exists() {
    if (!$this->exists) {
      $this->exists = $this->env->userExists($this->user_name);
    }
    return $this->exists;
  }

  /**
   * Get the resources.
   */
  public function resourcesGetAll() {
    return $this->resources;
  }

  /**
   * Get a resource.
   */
  public function resourceGet($name) {
    return isset($this->resources[$name]) ? $this->resources[$name] : FALSE;
  }

  /**
   * Create the resources.
   */
  public function resourcesCreate($resources) {
    if ($resources == '*' || $resources == array('*')) {
      foreach ($this->resources as $res) {
        $res->create();
      }
    }
    else {
      foreach ($resources as $res) {
        if (isset($this->resources[$res])) {
          $this->resources[$res]->create();
        }
        else {
          throw new Exception("Attempting to create resource '$res' which is not a recognized resource for this user");
        }
      }
    }
  }

  /**
   * Sync resources to another user.
   */
  public function syncTo(User $target, $resources) {
    $target_resources = $target->resourcesGetAll();

    foreach ($this->resources as $key => $res) {
      if ($resources == '*' || in_array($key, $resources)) {
        $res->syncTo($target_resources[$key]);
      }
    }
  }

  /**
   * Lazy load the password.
   */
  public function passwdGet() {
    if (empty($this->password)) {
      $this->password = $this->env->userPasswdGet($this->user_name);
    }
    return empty($this->password) ? twcore_passwd_gen() : $this->password;
  }

  /**
   * Login status.
   */
  public function isLoggedIn() {
    return $this->env->userIsLoggedIn($this->user_name);
  }

  /**
   * Gather this user's status into an array representation.
   */
  public function get() {
    $user = array(
      'user_name' => $this->user_name,
      'password' => $this->passwdGet(),
      'logged_in' => $this->isLoggedIn(),
    );
    foreach ($this->resources as $name => $resource) {
      $user['resources'][$name] = $resource->get();
    }
    return $user;
  }
}